@@ -1,5 +1,5 @@
 /*
-**	Text to Doc converter for 3Com PalmPilots
+**	Text to Doc converter for Palm Pilots
 **	txt2pdbdoc.c
 **
 **	Copyright (C) 1998  Paul J. Lucas
@@ -20,103 +20,91 @@
 */
 
 /* standard */
-#include <limits.h>
+#include <sys/types.h>			/* for FreeBSD */
+#include <netinet/in.h>			/* for htonl, etc */
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
-/*****************************************************************************
- *
- *	Define integral type Byte, Word, and DWord to match those on the
- *	Pilot being 8, 16, and 32 bits, respectively.
- *
- *****************************************************************************/
+/* local */
+#include "palm.h"
 
-#define U8_MAX		255u		/* max  8-bit unsigned */
-#if	UCHAR_MAX == U8_MAX
-typedef unsigned char	Byte;
-#else
-#error machine does not seem to support an 8-bit integral type
+/* types */
+#ifdef	bool
+#undef	bool
 #endif
+#define	bool		int
 
-#define	U16_MAX		65535u		/* max 16-bit unsigned */
-#if	USHRT_MAX == U16_MAX
-typedef unsigned short	Word;
-#else
-#error machine does not seem to support a 16-bit integral type
+#ifdef	false
+#undef	false
 #endif
+#define	false		0
 
-#define	U32_MAX		4294967295ul	/* max 32-bit unsigned */
-#if	USHRT_MAX == U32_MAX
-typedef unsigned short	DWord;
-#elif	UINT_MAX  == U32_MAX
-typedef unsigned int	DWord;
-#elif	ULONG_MAX == U32_MAX
-typedef unsigned long	DWord;
-#else
-#error machine does not seem to support a 32-bit integral type
+#ifdef	true
+#undef	true
 #endif
+#define	true		1
 
-/********** You shouldn't have to change anything below this line. ***********/
-
-#define bool		int
-#define false		0
-#define true		1
-
-#define BUFFER_SIZE	6000	/* why this is this value, I don't know */
-#define COUNT_BITS	3	/* ditto */
-#define DISP_BITS	11	/* ditto */
+/* constants */
+#define	BUFFER_SIZE	6000		/* big enough for uncompressed record */
+#define	COMPRESSED	2
+#define	COUNT_BITS	3		/* why this value?  I don't know */
+#define	DISP_BITS	11		/* ditto */
 #define	DOC_CREATOR	"REAd"
 #define	DOC_TYPE	"TEXt"
-#define RECORD_SIZE_MAX	4096	/* Pilots have a fixed 4K record size */
-#define VERSION		"1.2.1"
+#define	UNCOMPRESSED	1
+
+/* exit status codes */
+enum {
+	Exit_Success			= 0,
+	Exit_Usage			= 1,
+	Exit_No_Open_Source		= 2,
+	Exit_No_Open_Dest		= 3,
+	Exit_No_Read			= 4,
+	Exit_No_Write			= 5,
+	Exit_Not_Doc_File		= 6,
+	Exit_Unknown_Compression	= 7
+};
+
+/* macros */
+#define	NEW_BUFFER(b)	(b)->data = malloc( (b)->len = BUFFER_SIZE )
+
+#define	GET_Word(f,n) \
+	{ if ( fread( &n, 2, 1, f ) != 1 ) read_error(); n = ntohs(n); }
+
+#define	GET_DWord(f,n) \
+	{ if ( fread( &n, 4, 1, f ) != 1 ) read_error(); n = ntohl(n); }
+
+#define	PUT_Word(f,n) \
+	{  Word t = htons(n); if ( fwrite( &t, 2, 1, f ) != 1 ) write_error(); }
+
+#define	PUT_DWord(f,n) \
+	{ DWord t = htonl(n); if ( fwrite( &t, 4, 1, f ) != 1 ) write_error(); }
+
+#define	SEEK_REC_ENTRY(f,i) \
+	fseek( f, DatabaseHdrSize + RecordEntrySize * (i), SEEK_SET )
 
 /*****************************************************************************
  *
- *	PDB file header
- *
- *****************************************************************************/
-
-#define	dmDBNameLength	32	/* 31 chars + 1 null terminator */
-
-typedef struct {		/* 78 bytes total */
-	char	name[ dmDBNameLength ];
-	Word	attributes;
-	Word	version;
-	DWord	create_time;
-	DWord	modify_time;
-	DWord	backup_time;
-	DWord	modificationNumber;
-	DWord	appInfoID;
-	DWord	sortInfoID;
-	char	type[4];
-	char	creator[4];
-	DWord	id_seed;
-	DWord	nextRecordList;
-	Word	numRecords;
-} pdb_header;
-
+ * SYNOPSIS
+ */
+	struct doc_record0		/* 16 bytes total */
 /*
-** Some compilers pad structures out to DWord boundaries so using sizeof()
-** doesn't give the intended result.
-*/
-#define PDB_HEADER_SIZE		78
-#define	PDB_RECORD_HEADER_SIZE	8
-
-/*****************************************************************************
+ * DESCRIPTION
  *
- *	Doc record 0
+ *	Record 0 of a Doc file contains information about the document as a
+ *	whole.
  *
  *****************************************************************************/
-
-typedef struct {		/* 16 bytes total */
-	Word	version;	/* 1 = plain text, 2 = compressed */
+{
+	Word	version;		/* 1 = plain text, 2 = compressed */
 	Word	reserved1;
-	DWord	doc_size;	/* in bytes, when uncompressed */
-	Word	numRecords; 	/* text rec's only; = pdb_header.numRecords-1 */
-	Word	rec_size;	/* usually RECORD_SIZE_MAX */
+	DWord	doc_size;		/* in bytes, when uncompressed */
+	Word	num_records; 		/* PDB header numRecords - 1 */
+	Word	rec_size;		/* usually RECORD_SIZE_MAX */
 	DWord	reserved2;
-} doc_record0;
+};
+typedef struct doc_record0 doc_record0;
 
 /*****************************************************************************
  *
@@ -125,42 +113,30 @@ typedef struct {		/* 16 bytes total */
  *****************************************************************************/
 
 typedef struct {
-	Byte	*buf;
+	Byte	*data;
 	unsigned len;
 } buffer;
 
-char const*	me;				/* executable name */
+char const*	me;			/* executable name */
 
-bool		binary_opt = true;
+bool		binary_opt   = true;
 bool		compress_opt = true;
-bool		verbose_opt = false;
-Word		(*swap_Word )(  Word );
-DWord		(*swap_DWord)( DWord );
+bool		verbose_opt  = false;
 
 void		compress( buffer* );
-void		decode( char const *src_file_name, char const *dest_file );
+void		decode( char const *src_file_name, char const *dest_file_name );
 void		encode( char const *document_name,
-			char const *src_file_name, char const *dest_file
+			char const *src_file_name, char const *dest_file_name
 		);
 Byte*		mem_find( Byte *t, int t_len, Byte *m, int m_len );
 FILE*		open_src_file ( char const *file_name );
 FILE*		open_dest_file( char const *file_name );
 void		put_byte( buffer*, Byte, bool *space );
+void		read_error();
 void		remove_binary( buffer* );
-void		select_swap();
-Word		swap_Word_12( Word );
-Word		swap_Word_21( Word );
-DWord		swap_DWord_1234( DWord );
-DWord		swap_DWord_4321( DWord );
 void		uncompress( buffer* );
 void		usage();
-void		zero_fill( char*, int len );
-
-#define NEW_BUFFER(b)	(b)->buf = malloc( (b)->len = BUFFER_SIZE )
-#define GET_Word(f,n)	{ fread( &n, 2, 1, f ); n = swap_Word ( n ); }
-#define GET_DWord(f,n)	{ fread( &n, 4, 1, f ); n = swap_DWord( n ); }
-#define	PUT_Word(f,n)	{  Word n1 = swap_Word ( n ); fwrite( &n1, 2, 1, f ); }
-#define	PUT_DWord(f,n)	{ DWord n1 = swap_DWord( n ); fwrite( &n1, 4, 1, f ); }
+void		write_error();
 
 /*****************************************************************************
  *
@@ -197,8 +173,6 @@ void		zero_fill( char*, int len );
 	me = strrchr( argv[0], '/' );		/* determine base name... */
 	me = me ? me + 1 : argv[0];		/* ...of executable */
 
-	select_swap();
-
 	/********** Process command-line options *****************************/
 
 	opterr = 1;
@@ -222,8 +196,8 @@ void		zero_fill( char*, int len );
 				break;
 
 			case 'V':
-				printf( "txt2pdbdoc %s\n", VERSION );
-				exit( 0 );
+				printf( PACKAGE " " VERSION "\n" );
+				exit( Exit_Success );
 
 			case '?': /* Bad option. */
 				usage();
@@ -232,13 +206,13 @@ void		zero_fill( char*, int len );
 
 	if ( decode_opt ) {
 		if ( argc < 1 || argc > 2 ) usage();
-		decode( argv[ 0 ], argc == 2 ? argv[ 1 ] : 0 );
+		decode( argv[0], argc == 2 ? argv[1] : 0 );
 	} else {
 		if ( argc != 3 ) usage();
-		encode( argv[ 0 ], argv[ 1 ], argv[ 2 ] );
+		encode( argv[0], argv[1], argv[2] );
 	}
 
-	exit( 0 );
+	exit( Exit_Success );
 }
 
 /*****************************************************************************
@@ -249,7 +223,8 @@ void		zero_fill( char*, int len );
 /*
  * DESCRIPTION
  *
- *	Replace the given buffer with a compressed version of itself.
+ *	Replace the given buffer with a compressed version of itself.  I don't
+ *	understand this algorithm.  I just cleaned up the code.
  *
  * PARAMETERS
  *
@@ -267,9 +242,9 @@ void		zero_fill( char*, int len );
 	Byte *tail;	/* 1 past the current test buffer */
 	Byte *end;	/* 1 past the end of the input buffer */
 
-	p = p_prev = head = buf_orig = b->buf;
+	p = p_prev = head = buf_orig = b->data;
 	tail = head + 1;
-	end = b->buf + b->len;
+	end = b->data + b->len;
 
 	NEW_BUFFER( b );
 	b->len = 0;
@@ -305,11 +280,13 @@ void		zero_fill( char*, int len );
 
 				/* for longer runs, issue a run-code */
 				/* issue space char if required */
-				if ( space )
-					b->buf[ b->len++ ] = ' ', space = false;
+				if ( space ) {
+					b->data[ b->len++ ] = ' ';
+					space = false;
+				}
 
-				b->buf[ b->len++ ] = 0x80 + ( compound >> 8 );
-				b->buf[ b->len++ ] = compound & 0xFF;
+				b->data[ b->len++ ] = 0x80 + ( compound >> 8 );
+				b->data[ b->len++ ] = compound & 0xFF;
 				head = tail - 1;/* and start again */
 			}
 			p_prev = buf_orig;	/* start search again */
@@ -324,27 +301,27 @@ void		zero_fill( char*, int len );
 	free( buf_orig );
 
 	if ( space )
-		b->buf[ b->len++ ] = ' ';	/* add left-over space */
+		b->data[ b->len++ ] = ' ';	/* add left-over space */
 
 	/* final scan to merge consecutive high chars together */
 	for ( i = j = 0; i < b->len; ++i, ++j ) {
-		b->buf[ j ] = b->buf[ i ];
+		b->data[ j ] = b->data[ i ];
 
 		/* skip run-length codes */
-		if ( b->buf[ j ] >= 0x80 && b->buf[ j ] < 0xC0 )
-			b->buf[ ++j ] = b->buf[ ++i ];
+		if ( b->data[ j ] >= 0x80 && b->data[ j ] < 0xC0 )
+			b->data[ ++j ] = b->data[ ++i ];
 
 		/* if we hit a high char marker, look ahead for another */
-		else if ( b->buf[ j ] == '\1' ) {
-			b->buf[ j + 1 ] = b->buf[ i + 1 ];
+		else if ( b->data[ j ] == '\1' ) {
+			b->data[ j + 1 ] = b->data[ i + 1 ];
 			while ( i + 2 < b->len &&
-				b->buf[ i + 2 ] == 1 && b->buf[ j ] < 8
+				b->data[ i + 2 ] == 1 && b->data[ j ] < 8
 			) {
-				b->buf[ j ]++;
-				b->buf[ j + b->buf[ j ] ] = b->buf[ i + 3 ];
+				b->data[ j ]++;
+				b->data[ j + b->data[ j ] ] = b->data[ i + 3 ];
 				i += 2;
 			}
-			j += b->buf[ j ];
+			j += b->data[ j ];
 			++i;
 		}
 	}
@@ -370,48 +347,49 @@ void		zero_fill( char*, int len );
  *
  *****************************************************************************/
 {
-	FILE		*fin, *fout;
-	pdb_header	header;
-	doc_record0	rec0;
-	int		compression;
-	int		num_records, rec_num;
 	buffer		buf;
-	DWord		file_size, offset;
+	int		compression;
+	DWord		file_size, offset, rec_size;
+	FILE		*fin, *fout;
+	DatabaseHdrType	header;
+	int		num_records, rec_num;
+	doc_record0	rec0;
 
-	/********** open files; ensure source is a Doc file ******************/
+	/********** open files, read header, ensure source is a Doc file *****/
 
 	fin = open_src_file( src_file_name );
 
-	fread( &header, PDB_HEADER_SIZE, 1, fin );
-	if ( strncmp( header.type,    DOC_TYPE,    sizeof header.type ) ||
+	if ( fread( &header, DatabaseHdrSize, 1, fin ) != 1 ||
+	     strncmp( header.type,    DOC_TYPE,    sizeof header.type ) ||
 	     strncmp( header.creator, DOC_CREATOR, sizeof header.creator )
 	) {
 		fprintf( stderr, "%s: %s is not a Doc file\n",
 			me, dest_file_name
 		);
-		exit( 3 );
+		exit( Exit_Not_Doc_File );
 	}
-	num_records = swap_Word( header.numRecords ) - 1; /* not inc'l rec 0 */
+	num_records = ntohs( header.recordList.numRecords ) - 1; /* w/o rec 0 */
 
 	fout = dest_file_name ? open_dest_file( dest_file_name ) : stdout;
 
 	/********** read record 0 ********************************************/
 
-	fseek( fin, PDB_HEADER_SIZE, SEEK_SET );/* start of index */
+	SEEK_REC_ENTRY( fin, 0 );
 	GET_DWord( fin, offset );		/* get offset of rec 0 */
 	fseek( fin, offset, SEEK_SET );
-	fread( &rec0, sizeof rec0, 1, fin );
+	if ( fread( &rec0, sizeof rec0, 1, fin ) != 1 )
+		read_error();
 
-	compression = swap_Word( rec0.version );
-	if ( compression != 1 && compression != 2 ) {
+	compression = ntohs( rec0.version );
+	if ( compression != COMPRESSED && compression != UNCOMPRESSED ) {
 		fprintf( stderr,
 			"%s: error: unknown file compression type: %d\n",
 			me, compression
 		);
-		exit( 3 );
+		exit( Exit_Unknown_Compression );
 	}
 
-	/********* read buffer ***********************************************/
+	/********* read Doc file record-by-record ****************************/
 
 	fseek( fin, 0, SEEK_END );
 	file_size = ftell( fin );
@@ -424,27 +402,30 @@ void		zero_fill( char*, int len );
 		DWord next_offset;
 
 		/* read the record offset */
-		fseek( fin, PDB_HEADER_SIZE +
-			PDB_RECORD_HEADER_SIZE * rec_num, SEEK_SET
-		);
+		SEEK_REC_ENTRY( fin, rec_num );
 		GET_DWord( fin, offset );
+
+		/* read the next record offset to compute the record size */
 		if ( rec_num < num_records ) {
-			fseek( fin, PDB_HEADER_SIZE +
-				PDB_RECORD_HEADER_SIZE * (rec_num + 1), SEEK_SET
-			);
+			SEEK_REC_ENTRY( fin, rec_num + 1 );
 			GET_DWord( fin, next_offset );
 		} else
 			next_offset = file_size;
+		rec_size = next_offset - offset;
 
+		/* read the record */
 		fseek( fin, offset, SEEK_SET );
-		buf.len = fread( buf.buf, 1, next_offset - offset, fin );
+		buf.len = fread( buf.data, 1, rec_size, fin );
+		if ( buf.len != rec_size )
+			read_error();
 
-		if ( compression == 2 )
+		if ( compression == COMPRESSED )
 			uncompress( &buf );
 
-		fwrite( buf.buf, buf.len, 1, fout );
+		if ( fwrite( buf.data, buf.len, 1, fout ) != 1 )
+			write_error();
 		if ( verbose_opt )
-			fprintf( stderr, " %d", num_records - rec_num - 1 );
+			fprintf( stderr, " %d", num_records - rec_num );
 	}
 	if ( verbose_opt )
 		putc( '\n', stderr );
@@ -477,9 +458,10 @@ void		zero_fill( char*, int len );
  *
  *****************************************************************************/
 {
+	DWord		date;
 	FILE		*fin, *fout;
 	DWord		file_size;
-	pdb_header	header;
+	DatabaseHdrType	header;
 	doc_record0	rec0;
 	buffer		buf;
 	int		num_records, rec_num;
@@ -492,38 +474,41 @@ void		zero_fill( char*, int len );
 
 	fseek( fin, 0, SEEK_END );
 	file_size = ftell( fin );
-	fseek( fin, 0, SEEK_SET );
 	num_records = file_size / RECORD_SIZE_MAX;
 	if ( (long)num_records * RECORD_SIZE_MAX < file_size )
 		++num_records;
 
 	/********** create and write header **********************************/
 
-	zero_fill( header.name, sizeof header.name );
+	bzero( header.name, sizeof header.name );
 	strncpy( header.name, document_name, sizeof header.name - 1 );
 	if ( strlen( document_name ) > sizeof header.name - 1 )
 		strncpy( header.name + sizeof header.name - 4, "...", 3 );
-	header.attributes		= 0;
-	header.version			= 0;
-	strncpy( (char*)&header.create_time, "\x06\xD1\x44\xAE", 4 );
-	strncpy( (char*)&header.modify_time, "\x06\xD1\x44\xAE", 4 );
-	header.backup_time		= 0;
-	header.modificationNumber	= 0;
-	header.appInfoID		= 0;
-	header.sortInfoID		= 0;
+	header.attributes			= 0;
+	header.version				= 0;
+	date = htonl( palm_date() );
+	memcpy( &header.creationDate,		&date, 4 );
+	date = htonl( palm_date() );
+	memcpy( &header.modificationDate,	&date, 4 );
+	header.lastBackupDate			= 0;
+	header.modificationNumber		= 0;
+	header.appInfoID			= 0;
+	header.sortInfoID			= 0;
 	strncpy( header.type,    DOC_TYPE,    sizeof header.type );
 	strncpy( header.creator, DOC_CREATOR, sizeof header.creator );
-	header.id_seed			= 0;
-	header.nextRecordList		= 0;
-	header.numRecords		= swap_Word( num_records + 1 );
+	header.uniqueIDSeed			= 0;
+	header.recordList.nextRecordListID	= 0;
+	header.recordList.numRecords		= htons( num_records + 1 );
 						/* +1 for rec 0 */
 
-	fwrite( &header, PDB_HEADER_SIZE, 1, fout );
+	fseek( fin, 0, SEEK_SET );
+	if ( fwrite( &header, DatabaseHdrSize, 1, fout ) != 1 )
+		write_error();
 
 	/********** write record offsets *************************************/
 
 	num_offsets = num_records + 1;		/* +1 for rec 0 */
-	offset = PDB_HEADER_SIZE + PDB_RECORD_HEADER_SIZE * num_offsets;
+	offset = DatabaseHdrSize + RecordEntrySize * num_offsets;
 	index = 0x40 << 24 | 0x6F8000;		/* dirty + unique ID */
 
 	PUT_DWord( fout, offset );		/* offset for rec 0 */
@@ -536,14 +521,15 @@ void		zero_fill( char*, int len );
 
 	/********** write record 0 *******************************************/
 
-	rec0.version	= swap_Word( compress_opt + 1 );
-	rec0.reserved1	= 0;
-	rec0.doc_size	= swap_DWord( file_size );
-	rec0.numRecords	= swap_Word( num_records );
-	rec0.rec_size	= swap_Word( RECORD_SIZE_MAX );
-	rec0.reserved2	= 0;
+	rec0.version		= htons( compress_opt + 1 );
+	rec0.reserved1		= 0;
+	rec0.doc_size		= htonl( file_size );
+	rec0.num_records	= htons( num_records );
+	rec0.rec_size		= htons( RECORD_SIZE_MAX );
+	rec0.reserved2		= 0;
 
-	fwrite( &rec0, sizeof rec0, 1, fout );
+	if ( fwrite( &rec0, sizeof rec0, 1, fout ) != 1 )
+		write_error();
 
 	/********** write text ***********************************************/
 
@@ -553,13 +539,13 @@ void		zero_fill( char*, int len );
 		int bytes_read;
 
 		offset = ftell( fout );
-		fseek( fout, PDB_HEADER_SIZE +
-			PDB_RECORD_HEADER_SIZE * rec_num, SEEK_SET
-		);
+		SEEK_REC_ENTRY( fout, rec_num );
 		PUT_DWord( fout, offset );
 
-		if ( !(bytes_read = fread( buf.buf, 1, RECORD_SIZE_MAX, fin )) )
+		if ( !(bytes_read = fread( buf.data, 1, RECORD_SIZE_MAX, fin )))
 			break;
+		if ( ferror( fin ) )
+			read_error();
 		buf.len = bytes_read;
 
 		if ( binary_opt )
@@ -568,7 +554,8 @@ void		zero_fill( char*, int len );
 			compress( &buf );
 
 		fseek( fout, offset, SEEK_SET );
-		fwrite( buf.buf, buf.len, 1, fout );
+		if ( fwrite( buf.data, buf.len, 1, fout ) != 1 )
+			write_error();
 
 		if ( !verbose_opt )
 			continue;
@@ -625,19 +612,19 @@ void		zero_fill( char*, int len );
 		** in with an ASCII char.
 		*/
 		if ( c >= 0x40 && c <= 0x7F ) {
-			b->buf[ b->len++ ] = c ^ 0x80;
+			b->data[ b->len++ ] = c ^ 0x80;
 			return;
 		}
-		b->buf[ b->len++ ] = ' ';	/* couldn't squeeze it in */
+		b->data[ b->len++ ] = ' ';	/* couldn't squeeze it in */
 	} else if ( c == ' ' ) {
 		*space = true;
 		return;
 	}
 
 	if ( c >= 1 && c <= 8 || c >= 0x80 )
-		b->buf[ b->len++ ] = '\1';
+		b->data[ b->len++ ] = '\1';
 
-	b->buf[ b->len++ ] = c;
+	b->data[ b->len++ ] = c;
 }
 
 /*****************************************************************************
@@ -657,65 +644,33 @@ void		zero_fill( char*, int len );
  *
  *****************************************************************************/
 {
-	register Byte *const new_buf = malloc( b->len );
+	register Byte *const new_data = malloc( b->len );
 	int i, j;
+
 	for ( i = j = 0; i < b->len; ++i ) {
-		if ( b->buf[ i ] < 9 )		/* discard really low ASCII */
+		if ( b->data[ i ] < 9 )		/* discard really low ASCII */
 			continue;
-		switch ( b->buf[ i ] ) {
+		switch ( b->data[ i ] ) {
 
 			case '\r':
-				if ( i < b->len - 1 && b->buf[ i + 1 ] == '\n' )
+				if ( i < b->len - 1 && b->data[ i+1 ] == '\n' )
 					continue;	/* CR+LF -> LF */
 				/* no break; */
 
 			case '\f':
-				new_buf[ j ] = '\n';
+				new_data[ j ] = '\n';
 				break;
 
 			default:
-				new_buf[ j ] = b->buf[ i ];
+				new_data[ j ] = b->data[ i ];
 		}
 		++j;
 	}
-	free( b->buf );
-	b->buf = new_buf;
+	free( b->data );
+	b->data = new_data;
 	b->len = j;
 }
 
-/*****************************************************************************
- *
- * SYNOPSIS
- */
-	void select_swap()
-/*
- * DESCRIPTION
- *
- *	Determine the "endianness" of the machine by copying byte strings
- *	into a union and seeing how they come out when looked at as integers;
- *	set pointers-to-function appropriately.
- *
- *****************************************************************************/
-{
-	union { char c[2];  Word n; }  w;
-	union { char c[4]; DWord n; } dw;
-
-	strncpy(  w.c, "\1\2",     2 );
-	strncpy( dw.c, "\1\2\3\4", 4 );
-
-	swap_Word  =	 w.n == 0x0102 ? swap_Word_12 :
-			 w.n == 0x0201 ? swap_Word_21 : 0;
-
-	swap_DWord =	dw.n == 0x01020304 ? swap_DWord_1234 :
-			dw.n == 0x04030201 ? swap_DWord_4321 : 0;
-
-	if ( swap_Word && swap_DWord )
-		return;
-
-	fprintf( stderr, "%s: couldn't select byte-swapping algorithm\n", me );
-	exit( 2 );
-}  
-
 /*****************************************************************************
  *
  * SYNOPSIS
@@ -732,31 +687,32 @@ void		zero_fill( char*, int len );
  *
  *****************************************************************************/
 {
-	Byte *const new_buf = malloc( BUFFER_SIZE );
+	Byte *const new_data = malloc( BUFFER_SIZE );
 	int i, j;
+
 	for ( i = j = 0; i < b->len; ) {
-		register unsigned c = b->buf[ i++ ];
+		register unsigned c = b->data[ i++ ];
 
 		if ( c >= 1 && c <= 8 )
 			while ( c-- )			/* copy 'c' bytes */
-				new_buf[ j++ ] = b->buf[ i++ ];
+				new_data[ j++ ] = b->data[ i++ ];
 
 		else if ( c <= 0x7F )			/* 0,09-7F = self */
-			new_buf[ j++ ] = c;
+			new_data[ j++ ] = c;
 
 		else if ( c >= 0xC0 )			/* space + ASCII char */
-			new_buf[ j++ ] = ' ', new_buf[ j++ ] = c ^ 0x80;
+			new_data[ j++ ] = ' ', new_data[ j++ ] = c ^ 0x80;
 
 		else {					/* 80-BF = sequences */
 			register int di, n;
-			c = (c << 8) + b->buf[ i++ ];
+			c = (c << 8) + b->data[ i++ ];
 			di = (c & 0x3FFF) >> COUNT_BITS;
 			for ( n = (c & ((1 << COUNT_BITS) - 1)) + 3; n--; ++j )
-				new_buf[ j ] = new_buf[ j - di ];
+				new_data[ j ] = new_data[ j - di ];
 		}
 	}
-	free( b->buf );
-	b->buf = new_buf;
+	free( b->data );
+	b->data = new_data;
 	b->len = j;
 }
 
@@ -779,47 +735,38 @@ FILE* open_src_file( char const *file_name ) {
 	FILE *f = fopen( file_name, "rb" );
 	if ( f ) return f;
 	fprintf( stderr, "%s: can not open %s for input\n", me, file_name );
-	exit( 2 );
+	exit( Exit_No_Open_Source );
 }
 
 FILE* open_dest_file( char const *file_name ) {
 	FILE *f = fopen( file_name, "wb" );
 	if ( f ) return f;
 	fprintf( stderr, "%s: can not open %s for output\n", me, file_name );
-	exit( 2 );
+	exit( Exit_No_Open_Dest );
 }
 
-Word swap_Word_12( Word r ) {
-	return r;  
-}
-
-Word swap_Word_21( Word r ) {
-	return (r >> 8) | (r << 8);
-}
-
-DWord swap_DWord_1234( DWord r ) {
-	return r;
-}
-
-DWord swap_DWord_4321( DWord r ) {
-	return	( (r >> 24) & 0x00FF ) |   (r << 24) |
-		( (r >>  8) & 0xFF00 ) | ( (r <<  8) & 0xFF0000 );
+void read_error() {
+	fprintf( stderr, "%s: reading failed\n", me );
+	exit( Exit_No_Read );
 }
 
 void usage() {
-	fprintf( stderr, "\
-usage: %s [-c] [-b] [-v] document-name file.txt file.pdb\n\
-       %s  -d  [-b] [-v] file.pdb [file.txt]\n\n\
- options:\n\
- --------\n\
-  -b: don't strip binary characters\n\
-  -c: don't compress Doc file\n\
-  -d: decode Doc file to text\n\
-  -v: verbose\n", me, me );
-	exit( 1 );
+	fprintf( stderr,
+	"usage: %s [-c] [-b] [-v] document_name file.txt file.pdb\n"
+	"       %s  -d  [-b] [-v] file.pdb [file.txt]\n"
+	"       %s  -V\n\n"
+	"options:\n"
+	"--------\n"
+	" -b: don't strip binary characters\n"
+	" -c: don't compress Doc file\n"
+	" -d: decode Doc file to text\n"
+	" -v: verbose\n"
+	" -V: print version and exit\n", me, me, me );
+	exit( Exit_Usage );
 }
 
-void zero_fill( register char *p, register int len ) {
-	while ( len-- > 0 )
-		*p++ = '\0';
+void write_error() {
+	fprintf( stderr, "%s: writing failed\n", me );
+	exit( Exit_No_Write );
 }
+
